@@ -16,7 +16,8 @@ import operator
 from typing import Type, Generic, Any, Iterable, Callable, Union, Dict, Tuple
 
 from . import conversion
-from .base import Readable, Subscribable, T, Connectable, Writable, S, LogicHandler
+from .base import Readable, Subscribable, T, Connectable, Writable, S, LogicHandler, UninitializedError
+from .datatypes import RangeFloat1, RangeUInt8, HSVFloat1, RGBFloat1, RGBUInt8
 
 
 class ExpressionBuilder(Connectable[T], metaclass=abc.ABCMeta):
@@ -47,8 +48,6 @@ class ExpressionBuilder(Connectable[T], metaclass=abc.ABCMeta):
     * ``<=`` (le)
     * ``>`` (gt)
     * ``>=`` (ge)
-    * ``and``
-    * ``or``
 
     Additionally, the following methods are provided:
 
@@ -211,18 +210,12 @@ class ExpressionBuilder(Connectable[T], metaclass=abc.ABCMeta):
         else:
             return NotImplemented
 
-    def __and__(self, other) -> "BinaryCastExpressionHandler":
+    def and_(self, other) -> "BinaryCastExpressionHandler":
         return BinaryCastExpressionHandler(bool, self, other, operator.and_)
 
-    def __rand__(self, other) -> "BinaryCastExpressionHandler":
-        return BinaryCastExpressionHandler(bool, other, self, operator.and_)
-
-    def __or__(self, other) -> "BinaryCastExpressionHandler":
+    def or_(self, other) -> "BinaryCastExpressionHandler":
         return BinaryCastExpressionHandler(bool, self, other, operator.or_)
 
-    def __ror__(self, other) -> "BinaryCastExpressionHandler":
-        return BinaryCastExpressionHandler(bool, other, self, operator.or_)
-
     def not_(self) -> "UnaryExpressionHandler":
         return UnaryCastExpressionHandler(bool, self, operator.not_)
 
@@ -247,8 +240,8 @@ class ExpressionWrapper(Readable[T], Subscribable[T], ExpressionBuilder, Generic
     def subscribe(self, subscriber: Writable[S], convert: Union[Callable[[T], S], bool] = False):
         return self.wrapped.subscribe(subscriber, convert)
 
-    def trigger(self, target: LogicHandler) -> LogicHandler:
-        return self.wrapped.trigger(target)
+    def trigger(self, target: LogicHandler, synchronous: bool = False) -> LogicHandler:
+        return self.wrapped.trigger(target, synchronous=synchronous)
 
 
 class ExpressionHandler(Readable[T], Subscribable[T], ExpressionBuilder, Generic[T], metaclass=abc.ABCMeta):
@@ -266,14 +259,17 @@ class ExpressionHandler(Readable[T], Subscribable[T], ExpressionBuilder, Generic
         self.type = type_
         for operand in operands:
             if isinstance(operand, Subscribable):
-                operand.trigger(self.on_change)
+                operand.trigger(self.on_change, synchronous=True)
 
     @abc.abstractmethod
     async def evaluate(self) -> T:
         pass
 
     async def on_change(self, _value, origin):
-        await self._publish(await self.evaluate(), origin)
+        try:
+            await self._publish_and_wait(await self.evaluate(), origin)
+        except UninitializedError:
+            pass
 
     async def read(self) -> T:
         return await self.evaluate()
@@ -347,7 +343,8 @@ class IfThenElse(ExpressionHandler, Generic[T]):
     have the same value type). If the `condition` evaluates to True, this object evaluates to the value of `then`,
     otherwise it evaluates to the value of `otherwise`.
     """
-    def __init__(self, condition: Union[bool, Readable[bool]], then: Union[T, Readable[T]], otherwise: Union[T, Readable[T]]):
+    def __init__(self, condition: Union[bool, Readable[bool]], then: Union[T, Readable[T]],
+                 otherwise: Union[T, Readable[T]]):
         super().__init__(then.type if isinstance(then, Connectable) else type(then), (condition, then, otherwise))
         self.condition: Readable[bool] = self._wrap_static_value(condition)
         self.then = self._wrap_static_value(then)
@@ -357,7 +354,7 @@ class IfThenElse(ExpressionHandler, Generic[T]):
         return (await self.then.read()) if (await self.condition.read()) else (await self.otherwise.read())
 
 
-def not_(a):
+def not_(a) -> Union[bool, UnaryCastExpressionHandler[bool]]:
     """
     Create a :class:`ExpressionHandler` that wraps a *Connectable* or static value and evaluates to ``not x`` for the
     value `x`.
@@ -369,6 +366,32 @@ def not_(a):
     return not a
 
 
+def and_(a, b) -> Union[bool, BinaryCastExpressionHandler[bool]]:
+    """
+    Create a :class:`ExpressionHandler` that wraps two *Connectables* or static values a,b and evaluates to
+    ``a and b``.
+
+    This is the workaround for Python's limitations on overriding the ``and`` operator.
+    """
+    if (not (isinstance(a, Readable) and isinstance(a, Subscribable))
+            and not (isinstance(b, Readable) and isinstance(b, Subscribable))):
+        return bool(a and b)
+    return BinaryCastExpressionHandler(bool, a, b, operator.and_)
+
+
+def or_(a, b) -> Union[bool, BinaryCastExpressionHandler[bool]]:
+    """
+    Create a :class:`ExpressionHandler` that wraps two *Connectables* or static values a,b and evaluates to
+    ``a or b``.
+
+    This is the workaround for Python's limitations on overriding the ``or`` operator.
+    """
+    if (not (isinstance(a, Readable) and isinstance(a, Subscribable))
+            and not (isinstance(b, Readable) and isinstance(b, Subscribable))):
+        return bool(a or b)
+    return BinaryCastExpressionHandler(bool, a, b, operator.or_)
+
+
 # #################### #
 # Type Inference Rules #
 # #################### #
@@ -382,6 +405,8 @@ TYPES_ADD: Dict[Tuple[Type, Type], Type] = {
     (int, bool): int,
     (bool, float): float,
     (float, bool): float,
+    (str, str): str,
+    (bytes, bytes): bytes,
     (datetime.date, datetime.timedelta): datetime.date,
     (datetime.datetime, datetime.timedelta): datetime.datetime,
     (datetime.timedelta, datetime.date): datetime.date,
@@ -412,10 +437,19 @@ TYPES_MUL: Dict[Tuple[Type, Type], Type] = {
     (int, bool): int,
     (bool, float): float,
     (float, bool): float,
+    (str, int): str,
+    (int, str): str,
+    (bytes, int): bytes,
+    (int, bytes): bytes,
     (datetime.timedelta, float): datetime.timedelta,
     (float, datetime.timedelta): datetime.timedelta,
     (datetime.timedelta, int): datetime.timedelta,
     (int, datetime.timedelta): datetime.timedelta,
+    (RangeFloat1, float): float,
+    (float, RangeFloat1): float,
+    (RangeFloat1, int): int,
+    (int, RangeFloat1): int,
+    (RangeFloat1, RangeFloat1): RangeFloat1
 }
 TYPES_FLOORDIV: Dict[Tuple[Type, Type], Type] = {
     (int, int): int,
