@@ -15,10 +15,12 @@ import datetime
 import logging
 import math
 import random
+import time
 import weakref
 from typing import List, Optional, Callable, Any, Type, Union, Tuple, Iterable, Generic
 
 from .base import Subscribable, LogicHandler, Readable, Writable, T, UninitializedError
+from .expressions import ExpressionWrapper
 
 logger = logging.getLogger(__name__)
 
@@ -124,7 +126,7 @@ class _AbstractScheduleTimer(Subscribable[None], metaclass=abc.ABCMeta):
             logger.info("Scheduling next execution of timer %s for %s", self, next_execution)
             await _logarithmic_sleep(next_execution)
             self.last_execution = next_execution
-            asyncio.create_task(self._publish(None, []))
+            self._publish(None, [])  # we use asynchronous publishing, so no worries about timing
 
     @abc.abstractmethod
     def _next_execution(self) -> Optional[datetime.datetime]:
@@ -441,13 +443,14 @@ class _DelayedBool(Subscribable[bool], Readable[bool], metaclass=abc.ABCMeta):
     * :class:`TPulse`: Each True value triggers a True pulse of exactly ``delay`` length, but only if no pulse is
         currently active (i.e. the current value is False; a pulse is not re-triggerable).
     """
+    type = bool
+
     def __init__(self, wrapped: Subscribable[bool], delay: datetime.timedelta):
-        self.type = bool
         super().__init__()
         self.delay = delay
         self._value = False
         self._change_task: Optional[asyncio.Task] = None
-        wrapped.trigger(self._update)
+        wrapped.trigger(self._update, synchronous=True)
 
     @abc.abstractmethod
     async def _update(self, value: bool, origin: List[Any]): ...
@@ -465,11 +468,30 @@ class _DelayedBool(Subscribable[bool], Readable[bool], metaclass=abc.ABCMeta):
         except asyncio.CancelledError:
             return
         self._value = value
-        await self._publish(value, origin)
+        self._publish(value, origin)
         self._change_task = None
 
+    @property
+    def EX(self) -> ExpressionWrapper:
+        return ExpressionWrapper(self)
+
 
 class TOn(_DelayedBool):
+    """
+    Power-up delay for bool-type *Subscribable* objects
+
+    This object wraps a *Subscribable* bool object and applies a power-up / turn-on delay to its value. I.e., a `False`
+    value is re-published immediately, a `True` value is delayed for the configured time period. If a `False` is
+    received during the turn-on delay period, the `True` value is superseded and the `TOn`'s value does not change to
+    `True` at all.
+
+    The `TOn` object is *Subscribable* and *Readable* for publishing and providing the delayed value. Thus, it can be
+    used like an :ref:`SHC Expression <expressions>`. To actually use it in an Expression, you can use the :attr:`EX`
+    property, which wraps the `TOn` object in a :class:`shc.expressions.ExpressionWrapper`.
+
+    :param wrapped: The *Subscribable* object to be wrapped for delaying its value
+    :param delay: The power-up delay time. E.g. `datetime.timedelta(seconds=5)`
+    """
     async def _update(self, value: bool, origin: List[Any]):
         if value and self._change_task is None:
             self._change_task = asyncio.create_task(self._set_delayed(True, origin))
@@ -479,10 +501,25 @@ class TOn(_DelayedBool):
                 self._change_task = None
             if self._value:
                 self._value = False
-                await self._publish(False, origin)
+                self._publish(False, origin)
 
 
 class TOff(_DelayedBool):
+    """
+    Turn-off delay for bool-type *Subscribable* objects
+
+    This object wraps a *Subscribable* bool object and applies a turn-off delay to its value. I.e., a `True`
+    value is re-published immediately, a `False` value is delayed for the configured time period. If a `True` is
+    received during the turn-off delay period, the `False` value is superseded and the `TOff`'s value does stay `True`
+    continuously.
+
+    The `TOff` object is *Subscribable* and *Readable* for publishing and providing the delayed value. Thus, it can be
+    used like an :ref:`SHC Expression <expressions>`. To actually use it in an Expression, you can use the :attr:`EX`
+    property, which wraps the `TOff` object in a :class:`shc.expressions.ExpressionWrapper`.
+
+    :param wrapped: The *Subscribable* object to be wrapped for delaying its value
+    :param delay: The turn-off delay time. E.g. `datetime.timedelta(seconds=5)`
+    """
     async def _update(self, value: bool, origin: List[Any]):
         if not value and self._change_task is None:
             self._change_task = asyncio.create_task(self._set_delayed(False, origin))
@@ -492,10 +529,26 @@ class TOff(_DelayedBool):
                 self._change_task = None
             if not self._value:
                 self._value = True
-                await self._publish(True, origin)
+                self._publish(True, origin)
 
 
 class TOnOff(_DelayedBool):
+    """
+    Resettable boolean-delay for *Subscribable* objects
+
+    This object wraps a *Subscribable* bool object and applies a delay to its value. It behaves like a series of a
+    :class:`TOn` and a :class:`TOff` timer with the same delay time: All value updates of the wrapped object will be
+    delayed. Albeit, in contrast to a :class:`Delay` timer, a value update can be aborted by sending the
+    contrary value during the delay period. Thus, short pulses in either direction (fast False→True→False or
+    True→False→True toggling) is suppressed in the `TOnOff`'s value.
+
+    Like all delay timers, the `TOnOff` object is *Subscribable* and *Readable* for publishing and providing the delayed
+    value. Thus, it can be used like an :ref:`SHC Expression <expressions>`. To actually use it in an Expression, you
+    can use the :attr:`EX` property, which wraps the `TOnOff` object in a :class:`shc.expressions.ExpressionWrapper`.
+
+    :param wrapped: The *Subscribable* object to be wrapped for delaying its value
+    :param delay: The delay time. E.g. `datetime.timedelta(seconds=5)`
+    """
     async def _update(self, value: bool, origin: List[Any]):
         if value == self._value and self._change_task is None:
             return
@@ -505,24 +558,51 @@ class TOnOff(_DelayedBool):
 
 
 class TPulse(_DelayedBool):
+    """
+    Non-retriggerable pulse generator for bool-typed *Subscribable* objects
+
+    This object wraps a *Subscribable* bool object and creates a fixed length pulse (`True` period) on each rising edge
+    (False→True) change of its value. The pulse is not retriggerable, i.e. it is not prolonged when a second raising
+    edge occurs during the pulse.
+
+    Like all delay timers, the `TPulse` object is *Subscribable* and *Readable* for publishing and providing the pulse
+    signal. Thus, it can be used like an :ref:`SHC Expression <expressions>`. To actually use it in an Expression, you
+    can use the :attr:`EX` property, which wraps the `TPulse` object in a :class:`shc.expressions.ExpressionWrapper`.
+
+    :param wrapped: The *Subscribable* object to be wrapped for delaying its value
+    :param delay: The pulse length. E.g. `datetime.timedelta(seconds=5)`
+    """
+    def __init__(self, wrapped: Subscribable[bool], delay: datetime.timedelta):
+        super().__init__(wrapped, delay)
+        self._prev_value = False
+
     async def _update(self, value: bool, origin: List[Any]):
-        if value and not self._value:
+        rising_edge = value and not self._prev_value
+        self._prev_value = value
+        if rising_edge and not self._value:
             self._change_task = asyncio.create_task(self._set_delayed(False, origin))
             self._value = True
-            await self._publish(True, origin)
+            self._publish(True, origin)
 
 
 class Delay(Subscribable[T], Readable[T], Generic[T]):
     """
     A *Readable* and *Subscribable* object which wraps another Subscribable object and re-publishes its updates after
-    the time interval specified by ``delay``. It also *provides* its state/value from ``delay`` time ago.
+    the time interval specified by ``delay``. It also *provides* the state/value from ``delay`` time ago via ``read()``
+    calls.
+
+    :param wrapped: The *Subscribable* object to be wrapped for delaying its value
+    :param delay: The delay time. E.g. `datetime.timedelta(seconds=5)`
+    :param initial_value: If given, the value which is returned by :meth:`read` before a value has been received from
+        the `wrapped` object (and passed the delay period). If not given, :meth:`read` raises an
+        :class:`shc.base.UninitializedError` in this case.
     """
     def __init__(self, wrapped: Subscribable[T], delay: datetime.timedelta, initial_value: Optional[T] = None):
         self.type = wrapped.type
         super().__init__()
         self.delay = delay
         self._value: Optional[T] = initial_value
-        wrapped.trigger(self._update)
+        wrapped.trigger(self._update, synchronous=True)
 
     async def _update(self, value: T, origin: List[Any]) -> None:
         asyncio.create_task(self.__set_delayed(value, origin))
@@ -539,9 +619,131 @@ class Delay(Subscribable[T], Readable[T], Generic[T]):
         changed = value != self._value
         logger.debug("Value %s for Delay %s is now active and published", value, self)
         self._value = value
-        await self._publish(value, origin)
+        self._publish(value, origin)
 
     async def read(self) -> T:
         if self._value is None:
             raise UninitializedError("{} is not initialized yet.", repr(self))
         return self._value
+
+    @property
+    def EX(self) -> ExpressionWrapper:
+        return ExpressionWrapper(self)
+
+
+class TimerSwitch(Subscribable[bool], Readable[bool]):
+    """
+    A helper to program something similar to those old-fashioned digital timer switch adaptors
+
+    A `TimerSwitch` is basically a bool Variable with a fancy constructor to let some timers set the variable to True or
+    False. Additionally, a `duration` mode is built in, which allows to specify a duration after which the `TimerSwitch`
+    switches off (is set to False) automatically – similar to the :class:`TPulse` timer (but with re-trigger-ability).
+
+    A simple usage example for switching on at 8:00 and off at 13:37, but 9:00 and 15:00 on weekends may look like::
+
+        timer_switch = TimerSwitch(on=[At(weekday=[1,2,3,4,5], hour=8),
+                                       At(weekday=[6,7], hour=9),
+                                   off=[At(weekday=[1,2,3,4,5], hour=13, minute=37),
+                                        At(weekday=[6,7], hour=15)]
+        my_ceiling_lights.connect(timer_switch)
+
+    To enable or disable the `TimerSwitch`, you may want to take a look at the :class:`shc.misc.BreakableSubscription`.
+
+    :param on: A list of timers which should trigger a 'switch on' action (i.e. set the `TimerSwitch`'s value to True
+        when triggering)
+    :param off: A list of timers which should trigger a 'switch off' action (i.e. set the `TimerSwitch`'s value to
+        False when triggering). Either this parameter or the `duration` (but not both) must be specified.
+    :param duration: Period of time, after which the `TimerSwitch` will be switched off automatically after each
+        'switch on' event. Must be used as an alternative to the `off` parameter.
+    :param duration_random: An optional (maximum) offset to add to or substract from the `duration` period. If
+        `duration` is used, a random timedelta between ``-duration_random`` and ``+duration_random`` will be added to
+        each individual 'on' period.
+    """
+    type = bool
+
+    def __init__(self, on: Iterable[Subscribable], off: Optional[Iterable[Subscribable]] = None,
+                 duration: Optional[datetime.timedelta] = None, duration_random: Optional[datetime.timedelta] = None):
+        super().__init__()
+        if not off and duration is None:
+            raise ValueError("Either 'off' timer specs or a 'duration' must be specified")
+        if off and duration is not None:
+            raise ValueError("'off' timer specs and a 'duration' cannot be specified at the same time")
+        if duration_random and duration is None:
+            raise ValueError("A 'duration_random' value does not make sense without a duration")
+
+        self.value = False
+        self.duration = duration
+        self.duration_random = duration_random
+        self.delay_task: Optional[asyncio.Task] = None
+
+        for on_timer in on:
+            on_timer.trigger(self._on, synchronous=True)
+        if off:
+            for off_timer in off:
+                off_timer.trigger(self._off, synchronous=True)
+
+    async def _on(self, _v, origin) -> None:
+        if not self.value:
+            self.value = True
+            self._publish(True, origin)
+        if self.duration is not None:
+            if self.delay_task is not None:
+                self.delay_task.cancel()
+            self.delay_task = asyncio.create_task(self._delayed_off(origin))
+            timer_supervisor.add_temporary_task(self.delay_task)
+
+    async def _off(self, _v, origin) -> None:
+        if self.value:
+            self.value = False
+            self._publish(False, origin)
+
+    async def _delayed_off(self, origin) -> None:
+        assert(self.duration is not None)
+        await _logarithmic_sleep(datetime.datetime.now().astimezone() + self.duration
+                                 + _random_time(self.duration_random))
+        await self._off(False, origin)
+
+    async def read(self) -> bool:
+        return self.value
+
+    @property
+    def EX(self) -> ExpressionWrapper:
+        return ExpressionWrapper(self)
+
+
+class RateLimitedSubscription(Subscribable[T], Generic[T]):
+    """
+    A transparent wrapper for `Subscribable` objects, that delays and drops values to make sure that a given maximum
+    rate of new values is not exceeded.
+
+    :param wrapped: The Subscribable object to be wrapped
+    :param min_interval: The minimal allowed interval between published values in seconds
+    """
+    def __init__(self, wrapped: Subscribable[T], min_interval: float):
+        self.type = wrapped.type
+        super().__init__()
+        wrapped.trigger(self._new_value, synchronous=True)
+        self.min_interval = min_interval
+        self._last_publish = 0.0
+        self._delay_task: Optional[asyncio.Task] = None
+        self._latest_value: T
+        self._latest_origin: List[Any]
+
+    async def _new_value(self, value: T, origin: List[Any]) -> None:
+        next_allowed_publish = self._last_publish + self.min_interval
+        now = time.time()
+        if now >= next_allowed_publish:
+            self._last_publish = now
+            self._publish(value, origin)
+        else:
+            self._latest_value = value
+            self._latest_origin = origin
+            if not self._delay_task:
+                self._delay_task = asyncio.create_task(self._delayed_publish(next_allowed_publish - now))
+                timer_supervisor.add_temporary_task(self._delay_task)
+
+    async def _delayed_publish(self, delay: float):
+        await asyncio.sleep(delay)
+        self._last_publish = time.time()
+        self._delay_task = None
+        self._publish(self._latest_value, self._latest_origin)
